@@ -1,10 +1,10 @@
-/** 
+/**
  * XML Security Library example: Decrypting an encrypted file using a single key.
- * 
+ *
  * Decrypts encrypted XML file using a single DES key from a binary file
- * 
- * Usage: 
- *      ./decrypt1 <xml-enc> <des-key-file> 
+ *
+ * Usage:
+ *      ./decrypt1 <xml-enc> <des-key-file>
  *
  * Example:
  *      ./decrypt1 encrypt1-res.xml deskey.bin
@@ -12,8 +12,8 @@
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
- * 
- * Copyright (C) 2002-2016 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ *
+ * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 #include <stdlib.h>
 #include <string.h>
@@ -35,7 +35,7 @@
 
 int decrypt_file(const char* enc_file, const char* key_file);
 
-int 
+int
 main(int argc, char **argv) {
 #ifndef XMLSEC_NO_XSLT
     xsltSecurityPrefsPtr xsltSecPrefs = NULL;
@@ -55,22 +55,22 @@ main(int argc, char **argv) {
     xmlLoadExtDtdDefaultValue = XML_DETECT_IDS | XML_COMPLETE_ATTRS;
     xmlSubstituteEntitiesDefault(1);
 #ifndef XMLSEC_NO_XSLT
-    xmlIndentTreeOutput = 1; 
+    xmlIndentTreeOutput = 1;
 #endif /* XMLSEC_NO_XSLT */
 
     /* Init libxslt */
 #ifndef XMLSEC_NO_XSLT
     /* disable everything */
-    xsltSecPrefs = xsltNewSecurityPrefs(); 
+    xsltSecPrefs = xsltNewSecurityPrefs();
     xsltSetSecurityPrefs(xsltSecPrefs,  XSLT_SECPREF_READ_FILE,        xsltSecurityForbid);
     xsltSetSecurityPrefs(xsltSecPrefs,  XSLT_SECPREF_WRITE_FILE,       xsltSecurityForbid);
     xsltSetSecurityPrefs(xsltSecPrefs,  XSLT_SECPREF_CREATE_DIRECTORY, xsltSecurityForbid);
     xsltSetSecurityPrefs(xsltSecPrefs,  XSLT_SECPREF_READ_NETWORK,     xsltSecurityForbid);
     xsltSetSecurityPrefs(xsltSecPrefs,  XSLT_SECPREF_WRITE_NETWORK,    xsltSecurityForbid);
-    xsltSetDefaultSecurityPrefs(xsltSecPrefs); 
-#endif /* XMLSEC_NO_XSLT */                
+    xsltSetDefaultSecurityPrefs(xsltSecPrefs);
+#endif /* XMLSEC_NO_XSLT */
+
 
-                
     /* Init xmlsec library */
     if(xmlSecInit() < 0) {
         fprintf(stderr, "Error: xmlsec initialization failed.\n");
@@ -85,7 +85,7 @@ main(int argc, char **argv) {
 
     /* Load default crypto engine if we are supporting dynamic
      * loading for xmlsec-crypto libraries. Use the crypto library
-     * name ("openssl", "nss", etc.) to load corresponding 
+     * name ("openssl", "nss", etc.) to load corresponding
      * xmlsec-crypto library.
      */
 #ifdef XMLSEC_CRYPTO_DYNAMIC_LOADING
@@ -93,7 +93,7 @@ main(int argc, char **argv) {
         fprintf(stderr, "Error: unable to load default xmlsec-crypto library. Make sure\n"
                         "that you have it installed and check shared libraries path\n"
                         "(LD_LIBRARY_PATH and/or LTDL_LIBRARY_PATH) environment variables.\n");
-        return(-1);     
+        return(-1);
     }
 #endif /* XMLSEC_CRYPTO_DYNAMIC_LOADING */
 
@@ -111,23 +111,23 @@ main(int argc, char **argv) {
 
     if(decrypt_file(argv[1], argv[2]) < 0) {
         return(-1);
-    }    
-    
+    }
+
     /* Shutdown xmlsec-crypto library */
     xmlSecCryptoShutdown();
-    
+
     /* Shutdown crypto library */
     xmlSecCryptoAppShutdown();
-    
+
     /* Shutdown xmlsec library */
     xmlSecShutdown();
 
     /* Shutdown libxslt/libxml */
 #ifndef XMLSEC_NO_XSLT
-    xsltCleanupGlobals();            
+    xsltCleanupGlobals();
 #endif /* XMLSEC_NO_XSLT */
     xmlCleanupParser();
-    
+
     return(0);
 }
 
@@ -136,18 +136,18 @@ main(int argc, char **argv) {
  * @enc_file:           the encrypted XML  file name.
  * @key_file:           the Triple DES key file.
  *
- * Decrypts the XML file #enc_file using DES key from #key_file and 
+ * Decrypts the XML file #enc_file using DES key from #key_file and
  * prints results to stdout.
  *
  * Returns 0 on success or a negative value if an error occurs.
  */
-int 
+int
 decrypt_file(const char* enc_file, const char* key_file) {
     xmlDocPtr doc = NULL;
     xmlNodePtr node = NULL;
     xmlSecEncCtxPtr encCtx = NULL;
     int res = -1;
-    
+
     assert(enc_file);
     assert(key_file);
 
@@ -155,14 +155,14 @@ decrypt_file(const char* enc_file, const char* key_file) {
     doc = xmlParseFile(enc_file);
     if ((doc == NULL) || (xmlDocGetRootElement(doc) == NULL)){
         fprintf(stderr, "Error: unable to parse file \"%s\"\n", enc_file);
-        goto done;      
+        goto done;
     }
-    
+
     /* find start node */
     node = xmlSecFindNode(xmlDocGetRootElement(doc), xmlSecNodeEncryptedData, xmlSecEncNs);
     if(node == NULL) {
         fprintf(stderr, "Error: start node not found in \"%s\"\n", enc_file);
-        goto done;      
+        goto done;
     }
 
     /* create encryption context, we don't need keys manager in this example */
@@ -178,9 +178,9 @@ decrypt_file(const char* enc_file, const char* key_file) {
         fprintf(stderr,"Error: failed to load des key from binary file \"%s\"\n", key_file);
         goto done;
     }
-    
+
     /* set key name to the file name, this is just an example! */
-    if(xmlSecKeySetName(encCtx->encKey, key_file) < 0) {
+    if(xmlSecKeySetName(encCtx->encKey, BAD_CAST key_file) < 0) {
         fprintf(stderr,"Error: failed to set key name for key from \"%s\"\n", key_file);
         goto done;
     }
@@ -190,33 +190,34 @@ decrypt_file(const char* enc_file, const char* key_file) {
         fprintf(stderr,"Error: decryption failed\n");
         goto done;
     }
-        
+
     /* print decrypted data to stdout */
     if(encCtx->resultReplaced != 0) {
         fprintf(stdout, "Decrypted XML data:\n");
         xmlDocDump(stdout, doc);
     } else {
-        fprintf(stdout, "Decrypted binary data (%d bytes):\n", xmlSecBufferGetSize(encCtx->result));
+        fprintf(stdout, "Decrypted binary data (" XMLSEC_SIZE_FMT " bytes):\n",
+            xmlSecBufferGetSize(encCtx->result));
         if(xmlSecBufferGetData(encCtx->result) != NULL) {
-            fwrite(xmlSecBufferGetData(encCtx->result), 
-                  1, 
+            fwrite(xmlSecBufferGetData(encCtx->result),
+                  1,
                   xmlSecBufferGetSize(encCtx->result),
                   stdout);
         }
     }
     fprintf(stdout, "\n");
-        
+
     /* success */
     res = 0;
 
-done:    
+done:
     /* cleanup */
     if(encCtx != NULL) {
         xmlSecEncCtxDestroy(encCtx);
     }
-    
+
     if(doc != NULL) {
-        xmlFreeDoc(doc); 
+        xmlFreeDoc(doc);
     }
     return(res);
 }
