@@ -6,7 +6,7 @@
  * distribution for preciese wording.
  *
  * Copyright (C) 2003 Cordys R&D BV, All rights reserved.
- * Copyright (C) 2003-2016 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  * Copyright (c) 2005-2006 Cryptocom LTD (http://www.cryptocom.ru).
  */
 /**
@@ -25,14 +25,15 @@
 #include <xmlsec/transforms.h>
 #include <xmlsec/errors.h>
 #include <xmlsec/dl.h>
-#include <xmlsec/xmltree.h>
 #include <xmlsec/private.h>
+#include <xmlsec/xmltree.h>
 
 #include <xmlsec/mscrypto/app.h>
 #include <xmlsec/mscrypto/crypto.h>
 #include <xmlsec/mscrypto/x509.h>
-#include "private.h"
 
+#include "private.h"
+#include "../cast_helpers.h"
 
 #define XMLSEC_CONTAINER_NAME_A "xmlsec-key-container"
 #define XMLSEC_CONTAINER_NAME_W L"xmlsec-key-container"
@@ -354,33 +355,41 @@ static xmlSecMSCryptoProviderInfo xmlSecMSCryptoProviderInfo_Random[] = {
  * Returns: 0 on success or a negative value otherwise.
  */
 int
-xmlSecMSCryptoGenerateRandom(xmlSecBufferPtr buffer, size_t size) {
+xmlSecMSCryptoGenerateRandom(xmlSecBufferPtr buffer, xmlSecSize size) {
     HCRYPTPROV hProv = 0;
+    DWORD dwSize;
     int ret;
+    int res = -1;
 
     xmlSecAssert2(buffer != NULL, -1);
     xmlSecAssert2(size > 0, -1);
 
     ret = xmlSecBufferSetSize(buffer, size);
     if(ret < 0) {
-        xmlSecInternalError2("xmlSecBufferSetSize", NULL,
-                             "size=%d", size);
-        return(-1);
+        xmlSecInternalError2("xmlSecBufferSetSize", NULL, "size=" XMLSEC_SIZE_FMT, size);
+        goto done;
     }
-
     hProv = xmlSecMSCryptoFindProvider(xmlSecMSCryptoProviderInfo_Random, NULL, CRYPT_VERIFYCONTEXT, FALSE);
     if (0 == hProv) {
         xmlSecInternalError("xmlSecMSCryptoFindProvider", NULL);
-        return(-1);
-    }
-    if (FALSE == CryptGenRandom(hProv, (DWORD)size, xmlSecBufferGetData(buffer))) {
-        xmlSecMSCryptoError("CryptGenRandom", NULL);
-        CryptReleaseContext(hProv,0);
-        return(-1);
+        goto done;
     }
 
-    CryptReleaseContext(hProv, 0);
-    return(0);
+    XMLSEC_SAFE_CAST_SIZE_TO_ULONG(size, dwSize, goto done, NULL);
+    if (FALSE == CryptGenRandom(hProv, dwSize, xmlSecBufferGetData(buffer))) {
+        xmlSecMSCryptoError("CryptGenRandom", NULL);
+        goto done;
+    }
+
+    /* success */
+    res = 0;
+
+done:
+    /* cleanup */
+    if (hProv != 0) {
+        CryptReleaseContext(hProv, 0);
+    }
+    return(res);
 }
 
 /**
@@ -392,19 +401,21 @@ xmlSecMSCryptoGenerateRandom(xmlSecBufferPtr buffer, size_t size) {
  * Returns the system error message for the give error code.
  */
 void
-xmlSecMSCryptoGetErrorMessage(DWORD dwError, xmlChar * out, xmlSecSize outSize) {
-    LPTSTR errorText = NULL;
-    DWORD ret;
+xmlSecMSCryptoGetErrorMessage(DWORD dwError, xmlChar * out, int outLen) {
 #ifndef UNICODE
     WCHAR errorTextW[XMLSEC_MSCRYPTO_ERROR_MSG_BUFFER_SIZE];
 #endif /* UNICODE */
+    LPTSTR errorText = NULL;
+    DWORD dwRet;
+    int ret;
 
     xmlSecAssert(out != NULL);
-    xmlSecAssert(outSize > 0);
+    xmlSecAssert(outLen > 0);
+    out[0] = '\0';
 
     /* Use system message tables to retrieve error text, allocate buffer on local
        heap for error text, don't use any inserts/parameters */
-    ret = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM
+    dwRet = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM
                       | FORMAT_MESSAGE_ALLOCATE_BUFFER
                       | FORMAT_MESSAGE_IGNORE_INSERTS,
                       NULL,
@@ -413,26 +424,22 @@ xmlSecMSCryptoGetErrorMessage(DWORD dwError, xmlChar * out, xmlSecSize outSize)
                       (LPTSTR)&errorText,
                       0,
                       NULL);
-    if((ret <= 0) || (errorText == NULL)) {
-        out[0] = '\0';
+    if((dwRet <= 0) || (errorText == NULL)) {
         goto done;
     }
 
 #ifdef UNICODE
-    ret = WideCharToMultiByte(CP_UTF8, 0, errorText, -1, out, outSize, NULL, NULL);
+    ret = WideCharToMultiByte(CP_UTF8, 0, errorText, -1, (LPSTR)out, outLen, NULL, NULL);
     if(ret <= 0) {
-        out[0] = '\0';
         goto done;
     }
 #else /* UNICODE */
     ret = MultiByteToWideChar(CP_ACP, 0, errorText, -1, errorTextW, XMLSEC_MSCRYPTO_ERROR_MSG_BUFFER_SIZE);
     if(ret <= 0) {
-        out[0] = '\0';
         goto done;
     }
-    ret = WideCharToMultiByte(CP_UTF8, 0, errorTextW, -1, out, outSize, NULL, NULL);
+    ret = WideCharToMultiByte(CP_UTF8, 0, errorTextW, -1, (LPSTR)out, outLen, NULL, NULL);
     if(ret <= 0) {
-        out[0] = '\0';
         goto done;
     }
 #endif /* UNICODE */
@@ -474,35 +481,7 @@ xmlSecMSCryptoErrorsDefaultCallback(const char* file, int line, const char* func
  */
 LPWSTR
 xmlSecMSCryptoConvertUtf8ToUnicode(const xmlChar* str) {
-    LPWSTR res = NULL;
-    int len;
-    int ret;
-
-    xmlSecAssert2(str != NULL, NULL);
-
-    /* call MultiByteToWideChar first to get the buffer size */
-    ret = MultiByteToWideChar(CP_UTF8, 0, (LPCCH)str, -1, NULL, 0);
-    if(ret <= 0) {
-        return(NULL);
-    }
-    len = ret + 1;
-
-    /* allocate buffer */
-    res = (LPWSTR)xmlMalloc(sizeof(WCHAR) * len);
-    if(res == NULL) {
-        xmlSecMallocError(sizeof(WCHAR) * len, NULL);
-        return(NULL);
-    }
-
-    /* convert */
-    ret = MultiByteToWideChar(CP_UTF8, 0, (LPCCH)str, -1, res, len);
-    if(ret <= 0) {
-        xmlFree(res);
-        return(NULL);
-    }
-
-    /* done */
-    return(res);
+    return(xmlSecWin32ConvertUtf8ToUnicode(str));
 }
 
 /**
@@ -513,37 +492,9 @@ xmlSecMSCryptoConvertUtf8ToUnicode(const xmlChar* str) {
  *
  * Returns: a pointer to newly allocated string (must be freed with xmlFree) or NULL if an error occurs.
  */
-xmlChar* 
+xmlChar*
 xmlSecMSCryptoConvertUnicodeToUtf8(LPCWSTR str) {
-    xmlChar * res = NULL;
-    int len;
-    int ret;
-
-    xmlSecAssert2(str != NULL, NULL);
-
-    /* call WideCharToMultiByte first to get the buffer size */
-    ret = WideCharToMultiByte(CP_UTF8, 0, str, -1, NULL, 0, NULL, NULL);
-    if(ret <= 0) {
-        return(NULL);
-    }
-    len = ret + 1;
-
-    /* allocate buffer */
-    res = (xmlChar*)xmlMalloc(sizeof(xmlChar) * len);
-    if(res == NULL) {
-        xmlSecMallocError(sizeof(xmlChar) * len, NULL);
-        return(NULL);
-    }
-
-    /* convert */
-    ret = WideCharToMultiByte(CP_UTF8, 0, str, -1, (LPSTR)res, len, NULL, NULL);
-    if(ret <= 0) {
-        xmlFree(res);
-        return(NULL);
-    }
-
-    /* done */
-    return(res);
+    return(xmlSecWin32ConvertUnicodeToUtf8(str));
 }
 
 /**
@@ -556,35 +507,7 @@ xmlSecMSCryptoConvertUnicodeToUtf8(LPCWSTR str) {
  */
 LPWSTR
 xmlSecMSCryptoConvertLocaleToUnicode(const char* str) {
-    LPWSTR res = NULL;
-    int len;
-    int ret;
-
-    xmlSecAssert2(str != NULL, NULL);
-
-    /* call MultiByteToWideChar first to get the buffer size */
-    ret = MultiByteToWideChar(CP_ACP, 0, str, -1, NULL, 0);
-    if(ret <= 0) {
-        return(NULL);
-    }
-    len = ret;
-
-    /* allocate buffer */
-    res = (LPWSTR)xmlMalloc(sizeof(WCHAR) * len);
-    if(res == NULL) {
-        xmlSecMallocError(sizeof(WCHAR) * len, NULL);
-        return(NULL);
-    }
-
-    /* convert */
-    ret = MultiByteToWideChar(CP_ACP, 0, str, -1, res, len);
-    if(ret <= 0) {
-        xmlFree(res);
-        return(NULL);
-    }
-
-    /* done */
-    return(res);
+    return(xmlSecWin32ConvertLocaleToUnicode(str));
 }
 
 /**
@@ -595,47 +518,9 @@ xmlSecMSCryptoConvertLocaleToUnicode(const char* str) {
  *
  * Returns: a pointer to newly allocated string (must be freed with xmlFree) or NULL if an error occurs.
  */
-xmlChar* 
+xmlChar*
 xmlSecMSCryptoConvertLocaleToUtf8(const char * str) {
-    LPWSTR strW = NULL;
-    xmlChar * res = NULL;
-    int len;
-    int ret;
-
-    xmlSecAssert2(str != NULL, NULL);
-
-    strW = xmlSecMSCryptoConvertLocaleToUnicode(str);
-    if(strW == NULL) {
-        return(NULL);
-    }
-
-    /* call WideCharToMultiByte first to get the buffer size */
-    ret = WideCharToMultiByte(CP_ACP, 0, strW, -1, NULL, 0, NULL, NULL);
-    if(ret <= 0) {
-        xmlFree(strW);
-        return(NULL);
-    }
-    len = ret + 1;
-
-    /* allocate buffer */
-    res = (xmlChar*)xmlMalloc(sizeof(xmlChar) * len);
-    if(res == NULL) {
-        xmlSecMallocError(sizeof(xmlChar) * len, NULL);
-        xmlFree(strW);
-        return(NULL);
-    }
-
-    /* convert */
-    ret = WideCharToMultiByte(CP_ACP, 0, strW, -1, (LPSTR)res, len, NULL, NULL);
-    if(ret <= 0) {
-        xmlFree(strW);
-        xmlFree(res);
-        return(NULL);
-    }
-
-    /* done */
-    xmlFree(strW);
-    return(res);
+    return(xmlSecWin32ConvertLocaleToUtf8(str));
 }
 
 /**
@@ -646,47 +531,9 @@ xmlSecMSCryptoConvertLocaleToUtf8(const char * str) {
  *
  * Returns: a pointer to newly allocated string (must be freed with xmlFree) or NULL if an error occurs.
  */
-char * 
+char *
 xmlSecMSCryptoConvertUtf8ToLocale(const xmlChar* str) {
-    LPWSTR strW = NULL;
-    char * res = NULL;
-    int len;
-    int ret;
-
-    xmlSecAssert2(str != NULL, NULL);
-
-    strW = xmlSecMSCryptoConvertUtf8ToUnicode(str);
-    if(strW == NULL) {
-        return(NULL);
-    }
-
-    /* call WideCharToMultiByte first to get the buffer size */
-    ret = WideCharToMultiByte(CP_ACP, 0, strW, -1, NULL, 0, NULL, NULL);
-    if(ret <= 0) {
-        xmlFree(strW);
-        return(NULL);
-    }
-    len = ret + 1;
-
-    /* allocate buffer */
-    res = (char*)xmlMalloc(sizeof(char) * len);
-    if(res == NULL) {
-        xmlSecMallocError(sizeof(char) * len, NULL);
-        xmlFree(strW);
-        return(NULL);
-    }
-
-    /* convert */
-    ret = WideCharToMultiByte(CP_ACP, 0, strW, -1, res, len, NULL, NULL);
-    if(ret <= 0) {
-        xmlFree(strW);
-        xmlFree(res);
-        return(NULL);
-    }
-
-    /* done */
-    xmlFree(strW);
-    return(res);
+    return(xmlSecWin32ConvertUtf8ToLocale(str));
 }
 
 /**
@@ -697,13 +544,9 @@ xmlSecMSCryptoConvertUtf8ToLocale(const xmlChar* str) {
  *
  * Returns: a pointer to newly allocated string (must be freed with xmlFree) or NULL if an error occurs.
  */
-xmlChar* 
+xmlChar*
 xmlSecMSCryptoConvertTstrToUtf8(LPCTSTR str) {
-#ifdef UNICODE
-    return xmlSecMSCryptoConvertUnicodeToUtf8(str);
-#else  /* UNICODE */
-    return xmlSecMSCryptoConvertLocaleToUtf8(str);
-#endif /* UNICODE */
+    return(xmlSecWin32ConvertTstrToUtf8(str));
 }
 
 /**
@@ -716,11 +559,7 @@ xmlSecMSCryptoConvertTstrToUtf8(LPCTSTR str) {
  */
 LPTSTR
 xmlSecMSCryptoConvertUtf8ToTstr(const xmlChar*  str) {
-#ifdef UNICODE
-    return xmlSecMSCryptoConvertUtf8ToUnicode(str);
-#else  /* UNICODE */
-    return xmlSecMSCryptoConvertUtf8ToLocale(str);
-#endif /* UNICODE */
+    return(xmlSecWin32ConvertUtf8ToTstr(str));
 }
 
 /********************************************************************
@@ -765,7 +604,7 @@ xmlSecMSCryptoFindProvider(const xmlSecMSCryptoProviderInfo * providers,
 
         /* check errors */
         dwLastError = GetLastError();
-        switch(dwLastError) {
+        switch(HRESULT_FROM_WIN32(dwLastError)) {
         case NTE_BAD_KEYSET:
             /* This error can indicate that a newly installed provider
              * does not have a usable key container yet. It needs to be
