@@ -21,7 +21,7 @@
 #include <string.h>
 
 #include <nss.h>
-#include <pk11func.h>
+#include <pk11pub.h>
 #include <keyhi.h>
 #include <hasht.h>
 
