@@ -50,7 +50,10 @@ var withMSCrypto = 0;
 var withMSCng = 0;
 var withLibXSLT = 1;
 var withIconv = 1;
+var withFTP = 1;
 var withNT4 = 1;
+var withFTP = 0; /* disable ftp by default */
+var withHTTP = 0; /* disable http by default */
 
 /* Win32 build options. */
 var buildUnicode = 1;
@@ -115,7 +118,9 @@ function usage()
 	txt += "              \"openssl-110\", \"openssl=300\", \"openssl-300\",\n";
 	txt += "              \"nss\", \"mscrypto\", \"mscng\" (\"" + withCrypto + "\");\n"
  	txt += "  xslt:       LibXSLT is used (" + (withLibXSLT? "yes" : "no")  + ")\n";
- 	txt += "  iconv:      Use the iconv library (" + (withIconv? "yes" : "no")  + ")\n";
+	txt += "  iconv:      Use the iconv library (" + (withIconv ? "yes" : "no") + ")\n";
+	txt += "  ftp:        Enable FTP support (" + (withFTP ? "yes" : "no") + ")\n";
+	txt += "  http:       Enable HTTP support (" + (withHTTP ? "yes" : "no") + ")\n";
  	txt += "  nt4:        Enable NT 4.0 support (" + (withNT4? "yes" : "no")  + ")\n";
 	txt += "\nWin32 build options, default value given in parentheses:\n\n";
 	txt += "  unicode:    Build Unicode version (" + (buildUnicode? "yes" : "no")  + ")\n";
@@ -140,7 +145,7 @@ function usage()
 	txt += "  lib:        Additional search path for the linker, particularily\n";
 	txt += "              where libxml library can be found (" + buildLib + ")\n";
 	txt += "\nCrypto options, default value given in parentheses:\n\n";
-	txt += "  with-openssl3-engines:    Enable dynamic loading of xmlsec-crypto libraries (" + (withOpenSSL3Engines ? "yes" : "no") + ")\n";	
+	txt += "  with-openssl3-engines:    Enable dynamic loading of xmlsec-crypto libraries (" + (withOpenSSL3Engines ? "yes" : "no") + ")\n";
 	WScript.Echo(txt);
 }
 
@@ -187,6 +192,8 @@ function discoverVersion()
 	vf.WriteLine("WITH_MSCNG=" + withMSCng);
 	vf.WriteLine("WITH_LIBXSLT=" + (withLibXSLT ? "1" : "0"));
 	vf.WriteLine("WITH_ICONV=" + (withIconv ? "1" : "0"));
+	vf.WriteLine("WITH_FTP=" + (withFTP ? "1" : "0"));
+	vf.WriteLine("WITH_HTTP=" + (withHTTP ? "1" : "0"));
 	vf.WriteLine("WITH_NT4=" + (withNT4 ? "1" : "0"));
 	vf.WriteLine("UNICODE=" + (buildUnicode? "1" : "0"));
 	vf.WriteLine("DEBUG=" + (buildDebug? "1" : "0"));
@@ -320,6 +327,10 @@ for (i = 0; (i < WScript.Arguments.length) && (error == 0); i++) {
 			withLibXSLT = strToBool(arg.substring(opt.length + 1, arg.length));
 		else if (opt == "iconv")
 			withIconv = strToBool(arg.substring(opt.length + 1, arg.length));
+		else if (opt == "ftp")
+			withFTP = strToBool(arg.substring(opt.length + 1, arg.length));
+		else if (opt == "http")
+			withHTTP = strToBool(arg.substring(opt.length + 1, arg.length));
 		else if (opt == "nt4")
 			withNT4 = strToBool(arg.substring(opt.length + 1, arg.length));
 		else if (opt == "unicode")
@@ -357,7 +368,7 @@ for (i = 0; (i < WScript.Arguments.length) && (error == 0); i++) {
 		else if (opt == "cruntime")
 			cruntime = arg.substring(opt.length + 1, arg.length);
 		else if (opt == "with-openssl3-engines")
-			withOpenSSL3Engines = strToBool(arg.substring(opt.length + 1, arg.length));		
+			withOpenSSL3Engines = strToBool(arg.substring(opt.length + 1, arg.length));
 		else
 			error = 1;
 	} else if (i == 0) {
@@ -458,6 +469,8 @@ txtOut += "       Use MSCrypto: " + boolToStr(withMSCrypto) + "\n";
 txtOut += "          Use MSCng: " + boolToStr(withMSCng) + "\n";
 txtOut += "        Use LibXSLT: " + boolToStr(withLibXSLT) + "\n";
 txtOut += "          Use iconv: " + boolToStr(withIconv) + "\n";
+txtOut += "        Support FTP: " + boolToStr(withFTP) + "\n";
+txtOut += "       Support HTTP: " + boolToStr(withHTTP) + "\n";
 txtOut += "     NT 4.0 support: " + boolToStr(withNT4) + "\n";
 txtOut += "\n";
 txtOut += "Win32 build configuration\n";
